# EXAMPLE BindEnergy_WithSolvation
# Calculate binding energy including solvation.
# The current scene must contain receptor and ligand as objects 1 and 2.
# Note that the structures must have been energy minimized before, and that
# this macro is provided as an example only. Results could for example be
# improved by tuning the 'surfcost' parameter, the entropic cost for exposed surface.
# A more sophisticated method will become availabe in the future as part of
# YASARA Structure, which already now includes the AutoDock method to calculate
# free energies of binding and dissociation constants.
if !Atoms
  # Load an example complex
#  LoadSce (MacroDir)/BindEnergyWithSolvation
ForceField AMBER14,SetPar=Yes
# Place the cell around the complex
Cell Auto,Extension=20
Boundary periodic
# Ensure that the surface resolution is high enough for solvation calculations
SurfPar Resolution=3,Molecular=Numeric
# Calculate complex energy
# Energies are calculated with net-charge 0 to avoid overestimation of net-charge
# effects when comparing binding energies of molecules with different charges.
ChargeObj all,0
# Start a simulation so that receptor and ligand are transferred into the coordinate
# system of the simulation cell and the surface area is not simply the sum of both.
Sim On
epotcomp = Energy
esolcoulomb,esolvdw = SolvEnergy
surfmol = Surf Molecular
# This is a guesstimate of the entropic cost of exposing one A^2 to the solvent.
# If you get negative binding energies, double this guesstimate to 1.3e0.
# (The water molecules in the first solvent shell have to reorganize and lose
#  conformational freedom, which decreases their entropy and thus costs energy.
#  The value below does not include VdW interaction energies and is thus larger
#  than e.g. the 0.03 kJ/(mol*A^2) sometimes mentioned in textbooks).
# We divide by 6.02214199e20 to obtain J and multiply with JToUnit
# to get the currently selected energy unit
surfcost=(0.65e0/6.02214199e20)*JToUnit
# Add solvation entropy, i.e. the cost 'surfcost' of exposing surface
esolcomp=esolcoulomb+esolvdw+surfmol*surfcost
# Calculate potential and solvation energy for each of the two
for i=1 to 2
  RemoveObj not (i)
  ChargeObj all,0
  epot(i) = Energy
  esolcoulomb,esolvdw = SolvEnergy
  surfmol = Surf Molecular
  esol(i)=esolcoulomb+esolvdw+surfmol*surfcost
  AddObj all
# Calculate result: energy of separated compounds - energy of complex
ebind=(epot1+epot2+esol1+esol2)-(epotcomp+esolcomp)
Print 'Binding energy is (0.000+ebind) (EnergyUnit)'
quit
