# YASARA MACRO
# TOPIC:       3. Molecular Dynamics
# TITLE:       Analyzing the ligand binding energy during a molecular dynamics simulation
# REQUIRES:    Dynamics
# AUTHOR:      Elmar Krieger
# LICENSE:     GPL
# DESCRIPTION: This macro calculates the binding energy of a ligand during a molecular dynamics simulation, including the time average. By default, the receptor must be object 1, the ligand must be object 2 (or change the settings). More positive energies indicate better binding, negative energies DO NOT indicate no binding, see the 'BindEnergy' command in the user manual for details.

# The structure to analyze must be present with a .sce extension.
# You can either set the target structure by clicking on Options > Macro > Set target,
# by providing it as command line argument (see docs at Essentials > The command line),
# or by uncommenting the line below and specifying it directly.
# MacroTarget = '/home/myprojects/1h8a/1h8a'

# Forcefield to use (these are all YASARA commands, so no '=' used)
# Use YASARA2 in YASARA Structure to include a quality Z-score
ForceField AMBER14,SetPar=Yes

# Set the object numbers of receptor and ligand for binding energy calculation.
# All other objects (water etc.) will be ignored. If the ligand is not in a separate
# object, set ligobj also to 1 and specify the ligand residue with 'ligres' below:
recobj=1
ligobj=1
# If the ligand is part of the receptor object, specify its residue ID, e.g. ligres='ATP 301'
ligres='DST 601'

# First snapshot to be analyzed, increase number to ignore an equilibration period.
# (By default, md_run.mcr saves snapshots every 25ps, choosing 40 thus starts the analysis after 1 nanosecond)
firstsnapshot=100

# Choose method, either boundary elements ('BoundaryFast') or Poisson-Boltzmann ('PBS').
# In the latter case you get almost 'MM/PBSA', just without the entropy term from normal mode analysis.
# When comparing with other MM/PBSA calculations, make sure to use the same 'surfcost' below.
method='PBS'

# This is a guesstimate of the entropic cost of exposing one A^2 to the solvent in kJ/mol
# (The water molecules in the first solvent shell have to reorganize and lose
#  conformational freedom, which decreases their entropy and thus costs energy.
#  The value below does not include VdW interaction energies and is thus larger
#  than e.g. the 0.03 kJ/(mol*A^2) sometimes mentioned in textbooks). 
# We divide by 6.02214199e20 to obtain J and multiply with JToUnit
# to get the currently selected energy unit
surfcost=(0.65e0/6.02214199e20)*JToUnit
  
# No change required below this point
# ===================================

# Do we have a target?
if MacroTarget==''
  RaiseError "This macro requires a target. Either edit the macro file or click Options > Macro > Set target to choose a target structure"

Clear
Console Off
# Ensure that the surface resolution is high enough for solvation calculations
SurfPar Resolution=3,Molecular=Numeric
  
# Load scene with water or other solvent
waterscene = FileSize (MacroTarget)_water.sce
solventscene = FileSize (MacroTarget)_solvent.sce
if waterscene
  LoadSce (MacroTarget)_water
elif solventscene 
  LoadSce (MacroTarget)_solvent
else
  RaiseError 'Could not find initial scene file (MacroTarget)_water.sce. You must run a simulation with the macro md_run first'

if ligres!=''
  # Ligand is part of receptor object, perform consistency checks
  residues = CountRes (ligres)
  if residues==0
    RaiseError 'The ligand residue (ligres) was not found'
  if residues>1
    RaiseError 'More than one ligand residue (ligres) was found'
  if ligobj!=recobj or recobj!=1
    RaiseError 'If a ligand residue is selected, ligand and receptor must both be in object 1'
  if Objects!=3
    RaiseError 'The scene must contain three objects: Receptor/Ligand, SimCell and Water'
  # Extract ligand residue
  SplitRes (ligres)
  SplitObj (recobj),Center=Yes,Res (ligres)
  ligobj=3
  # Are atoms following the ligand?
  first,last = SpanAtom Res (ligres)
  if last!=Atoms
    # Yes, also split those off
    SplitAtom (last+1)
    SplitObj 4,Center=Yes,Atom (last+1)
    recobj='1 4'
  # Put the water last so that original atom order is preserved and simulation snapshots match
  RenumberObj Water,(Objects+1)
  RenumberObj all,1

ShowMessage "Preparing analysis, please wait..."
Wait 1

# Create a table with 2 columns
MakeTab Final,Dimensions=2,Columns=2

# Determine trajectory format
for format in 'xtc','mdcrd','sim'
  found = FileSize (MacroTarget).(format)
  if found
    break
  
i=00000+firstsnapshot
emin=1e99
last=0
while !last
  # Load next snapshot from SIM, XTC or MDCRD trajectory
  if format=='sim'
    sim = FileSize (MacroTarget)(i).sim
    if not sim
      break
    LoadSim (MacroTarget)(i)
  else
    # Set last (end of file) to 1 if last snapshot loaded
    last = Load(format) (MacroTarget),(i+1)
  Sim Pause
  # Add time in picoseconds to table
  simtime = Time
  ShowMessage 'Analyzing snapshot (0+i) at (0+(simtime/1000)) ps'
  Wait 1
  Tabulate (simtime/1000)
#  for type in 'cmp','rec','lig'
  for type in 'rec'
    # Keep only the relevant objects
    if type=='cmp'
      RemoveObj not (recobj) (ligobj)
    else
      RemoveObj not ((type)obj)
    # Energies are calculated with net-charge 0 to avoid overestimation of net-charge
    # effects when comparing binding energies of molecules with different charges.
    ChargeObj all,0
    # Start a simulation so that receptor and ligand are transferred into the coordinate
    # system of the simulation cell and the surface area is not simply the sum of both.
    Sim on
    epot(type) = Energy
    esolcoulomb,esolvdw = SolvEnergy (method)
    molsurf = Surf molecular
    esol(type)=esolcoulomb+esolvdw+molsurf*surfcost
    print 'epot(type)=(epot(type)), (esolcoulomb),(esolvdw),surfcost=(molsurf*surfcost), esol(type)=(esol(type)), total=(epot(type)+esol(type))'
    AddObj all
  # Calculate result: energy of separated compounds - energy of complex
#  Tabulate (epotrec+esolrec+epotlig+esollig-epotcmp-esolcmp)
  Tabulate (epotrec+esolrec)
  # Next snapshot
  i=i+1

if i==firstsnapshot
  RaiseError "This macro is meant to analyze a molecular dynamics trajectory created with md_run, but none was found in this directory"

# Tabulate average binding energy
Tabulate 'Average'
vallist() = Tab Final,Column=2
Tabulate (mean vallist)
# Create a table header
header='____Time[ps] Energy[(EnergyUnit)]'
SaveTab Final,(MacroTarget)_pbs_energy,Format=Text,Columns=2,NumFormat=12.3f,(header)

HideMessage

# Exit YASARA if this macro was provided as command line argument in console mode
if runWithMacro and ConsoleMode
  Exit
